#-*- Mode: python; coding: utf-8; tab-width: 8; indent-tabs-mode: t; -*-

#
# Copyright (C) 2009 Utkarsh Upadhyay
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The Rhythmbox authors hereby grant permission for non-GPL compatible
# GStreamer plugins to be used and distributed together with GStreamer
# and Rhythmbox. This permission is above and beyond the permissions granted
# by the GPL license by which Rhythmbox is covered. If you modify this code
# you may extend this exception to your version of the code, but you are not
# obligated to do so. If you do not wish to do so, delete this exception
# statement from your version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
#

import rb, rhythmdb
import pygtk
import gtk

class ExtraPlaycountPlugin (rb.Plugin):
	def __init__(self):
		rb.Plugin.__init__(self)

	def activate(self, shell):
		self.shell = shell
		self.db = shell.get_property( 'db' )
		# Attaching to the event whenever Song Info window is created
		self.csi_id = shell.connect( 'create_song_info', self.createExtraInfo)

	def createExtraInfo(self, shell, songInfo, isMultiple):
		# Only add the pane if the song info box is for multiple songs
		if isMultiple is True:
			songInfo.append_page( _("Extra Play Count"), self.constructPane( songInfo ) )

	def constructPane(self, songInfo):
		# Makes the pane to attach to the Song Properties dialog
		paneBox = gtk.VBox( homogeneous = False, spacing = 0 )

		# Setting up the defaults
		self.chosenProperty = rhythmdb.PROP_ALBUM 
		self.targetProperty = rhythmdb.PROP_PLAY_COUNT

		# The text view contains the output.
		textView = gtk.TextView( buffer=None )
		textView.set_editable( True )
		textView.set_justification( gtk.JUSTIFY_CENTER )

		def updateText():
			# To update the text in the TextView
			# It is called everytime a radio button is toggled
			selectedEntries = songInfo.props.selected_entries
			textView.get_buffer().set_text( self.getPropertyInfo( selectedEntries ) )

		updateText()
		
		textPane = gtk.ScrolledWindow()
		textPane.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC )
		textPane.add( textView )

		# The optionFrame contains the radio buttons
		optionPane = gtk.HBox( homogeneous = False, spacing = 0 )

		keyOptionPane = gtk.VBox( homogeneous = True, spacing = 0 )
		valueOptionPane = gtk.VBox( homogeneous = True, spacing = 0 )

		def changeChosenCallback(widget, prop):
			if widget.get_active(): 
				self.chosenProperty = prop
				updateText()

		def changeTragetCallback(widget, value):
			if widget.get_active():
				self.targetProperty = value
				updateText()

		albumRadioButton = gtk.RadioButton( group = None, label = "A_lbum" )
		albumRadioButton.connect( "toggled", changeChosenCallback, rhythmdb.PROP_ALBUM )
		albumRadioButton.set_active( True )
		keyOptionPane.pack_start( albumRadioButton, expand = True, fill = True, padding = 0 )

		artistRadioButton = gtk.RadioButton( group = albumRadioButton, label = 'A_rtist' )
		artistRadioButton.connect( "toggled", changeChosenCallback, rhythmdb.PROP_ARTIST )
		keyOptionPane.pack_start( artistRadioButton, expand = True, fill = True, padding = 0 )

		playCountRadioButton = gtk.RadioButton( group = None, label = '_Play Count' )
		playCountRadioButton.set_active( True )
		playCountRadioButton.connect( "toggled", changeTragetCallback, rhythmdb.PROP_PLAY_COUNT )
		valueOptionPane.pack_start( playCountRadioButton, expand = True, fill = True, padding = 0 )

		occurrenceRadioButton = gtk.RadioButton( group = playCountRadioButton, label = '_Occurrence' )
		occurrenceRadioButton.connect( "toggled", changeTragetCallback, -1 )
		valueOptionPane.pack_start( occurrenceRadioButton, expand = True, fill = True, padding = 0 )

		optionPane.pack_start( keyOptionPane, expand = True, fill = True, padding = 0 )
		optionPane.pack_start( gtk.VSeparator(), expand = False, fill = True, padding = 0 )
		optionPane.pack_start( valueOptionPane, expand = True, fill = True, padding = 0 )

		# Constructing the final pane
		paneBox.pack_start( optionPane, expand = False, padding = 0 )
		paneBox.pack_start( gtk.VSeparator(), expand = False, padding = 0 )
		paneBox.pack_start( textPane, expand = True, fill = True, padding = 0 )

		return paneBox

	# Deals with the entries and returns the exact text to display in the textView
	def getPropertyInfo(self, entries ):
		# This is the function that updates the tuples stored in the albumCount hash
		def updateEntry( tup1, tup2 ):
			( playCount1, name1 ) = tup1
			( playCount2, name2 ) = tup2

			# If the first tuple is the default value, 
			if name1 == '':
				# Then update both name as well as playcount
				return ( playCount2, name2 )
			else: 
				# Otherwise, just update the playcount
				return ( playCount1 + playCount2, name1 )

		# Will keep a dictionary of { 'name_lower' -> (name_actual, play_count) }
		extraCount = {}

		for entry in entries:
			propName = self.db.entry_get( entry, self.chosenProperty )
			if self.targetProperty == rhythmdb.PROP_PLAY_COUNT:
				count = self.db.entry_get( entry, rhythmdb.PROP_PLAY_COUNT ) 
			else: 	count = 1
			extraCount[ propName.lower() ] = updateEntry( extraCount.get( propName.lower() , (0, '') ),  ( count, propName ) )

		countExtraTuples = extraCount.values()

		# Sorting according to the play count, 
		list.sort( countExtraTuples, reverse=True )

		# Calculate the total number of plays
		totalCount = reduce ( lambda x,(y,z) : x + y, countExtraTuples, 0 )

		if self.targetProperty == rhythmdb.PROP_PLAY_COUNT:	property = "Plays"
		else:							property = "Occurrences"

		return reduce( lambda x,(y,z): x + "\n" + z + " : " + str(y), countExtraTuples, "Total %s = %d\n" % (property, totalCount) )

	def deactivate(self, shell):
		# Disconnecting from the event
		shell.disconnect( self.csi_id )
		del self.shell
		del self.db


